Public Class AccessFuncs

    '*********************************************************************
    '
    '  GPIB Library for Visual BASIC .NET
    '
    '  File: ACCESS.VB                                        12-04-2006
    '
    ' This file is to provide ICS internal access to the GPIB-32.DLL config
    ' file. This allows read/write/modify/examine of the configuration.
    '
    ' This file is ONLY for use with the Visual Basic 8 from Microsoft's 
    ' Visual Studio 2005.
    '
    '***********************************************************************

    'COMMON SHARED /GPIBVARS/ IBSTA%, IBERR%, IBCNT%, IBCNTL&
    Public Shared Ibsta%, Iberr%, Ibcnt%, Ibcntl&
    Public Shared AccessInProgress%
    Public Shared Sn As String

    ' Board Types
    Public Const USB1_type As Integer = 8       ' 488-USB1
    Public Const USB2_type As Integer = &H102   ' 488-USB2
    Public Const LPCI_type As Integer = &H103   ' Adlink 488-LPCI, 488-PXI
    Public Const Max_Bds As Integer = 15        'max board count -1

    ' Error codes returned to IBERR variable
    Public Const EDIR As Integer = 1000     ' Missing GPIBDIR environment variable
    Public Const EFIL As Integer = 1001     ' Error opening file (missing or write protected)
    Public Const ERED As Integer = 1002     ' Read error on GPIB.INI file
    Public Const EWRT As Integer = 1003     ' Write error on GPIB.INI file
    Public Const EINI As Integer = 1004     ' GPIB.INI file not loaded (try loading it)


    ' The following constants are used with ibconfig() to select a
    ' configurable option. 0 = record not used, 1 = record used
    Public Const IbaDEFINED As Integer = 1000        ' Get Board record defined/used 
    Public Const IbcDEFINED As Integer = 1000        ' Set Board record defined/used


    ' GPIB-32.DLL routines
    ' BrdDev = 0 if device, = nonzero if board
    ' mode% = 0 for read, = nonzero or 1 for write

    'Finds physical device. BD# = driver#.  Returns Sn$
    Declare Function USB1FindDev32 Lib "Access.dll" Alias "_488USB1FindDevice" (ByVal board%, ByVal Sn$) As Integer
    Declare Function USB2FindDev32 Lib "Access.dll" Alias "_488USB2FindDevice" (ByVal board%, ByVal Sn$) As Integer
    Declare Function LPCIFindDev32 Lib "Access.dll" Alias "_488PciFindDevice" (ByVal board%, ByVal Sn$) As Integer

    'Read/Write USB Device EEPROM - Use in production only
    Declare Function IbEeprom32 Lib "Access.dll" Alias "IbEeprom" (ByVal board%, ByVal mode%, ByVal eeprom$) As Integer

    'File access functions. BD# = file record#
    Declare Function Ibask32 Lib "Access.dll" Alias "Ibask" (ByVal BrdDev%, ByVal board%, ByVal item%, ByRef value%) As Integer
    Declare Function Ibconfig32 Lib "Access.dll" Alias "Ibconfig" (ByVal BrdDev%, ByVal board%, ByVal item%, ByVal value%) As Integer
    'Creates file in RAM,saves a copy to disk.  mode 1 = write, 0 = read
    Declare Function Ibcreate32 Lib "Access.dll" Alias "Ibcreate" () As Integer
    Declare Function Ibload32 Lib "Access.dll" Alias "Ibload" () As Integer
    Declare Function Ibsave32 Lib "Access.dll" Alias "Ibsave" () As Integer
    Declare Function Ibname32 Lib "Access.dll" Alias "Ibname" (ByVal BrdDev%, ByVal board%, ByVal mode%, ByVal name$) As Integer 
    Declare Function IbSn32 Lib "Access.dll" Alias "IbSn" (ByVal board%, ByVal mode%, ByVal sn$) As Integer

    Declare Function GetIbsta32 Lib "Access.dll" Alias "GetIbsta" () As Integer
    Declare Function GetIberr32 Lib "Access.dll" Alias "GetIberr" () As Integer
    Declare Function GetIbcnt32 Lib "Access.dll" Alias "GetIbcnt" () As Integer
    Declare Function GetIbcntl32 Lib "Access.dll" Alias "GetIbcntl" () As Long

    Declare Sub LoadGPIBConfig32 Lib "GPIB-32.dll" Alias "LoadGPIBConfig" ()

    Declare Function TestGpibDirEnvVar32 Lib "Access.dll" Alias "TestGpibDirEnvVar" () As Integer
    Declare Function TestGpibIniFile32 Lib "Access.dll" Alias "TestGpibIniFile" () As Integer
    Declare Function TestGpibLib32 Lib "Access.dll" Alias "TestGpibLib" () As Integer



    '************************
    ' Functions
    '************************

    Public Shared Sub copy_globals()
        Ibsta% = GetIbsta32()
        Iberr% = GetIberr32()
        Ibcnt% = GetIbcnt32()
        Ibcntl& = 0
        Ibcntl& = GetIbcntl32() And &HFFFFFFFF&
    End Sub


    Public Shared Function USB1FindDev(ByVal board%, ByRef Sn$) As Integer
        USB1FindDev = USB1FindDev32(board%, Sn$)
        Call copy_globals()
    End Function

    Public Shared Function USB2FindDev(ByVal board%, ByRef Sn$) As Integer
        USB2FindDev = USB2FindDev32(board%, Sn$)
        Call copy_globals()
    End Function

    Public Shared Function LPCIFindDev(ByVal board%, ByRef Sn$) As Integer
        LPCIFindDev = LPCIFindDev32(board%, Sn$)
        Call copy_globals()
    End Function

    Public Shared Function Ibask(ByVal BrdDev%, ByVal board%, ByVal item%, ByRef value%) As Integer
        Ibask = Ibask32(BrdDev%, board%, item%, value%)
        Call copy_globals()
    End Function

    ' BrdDev = 0 if device, = nonzero if board
    Public Shared Function Ibconfig(ByVal BrdDev%, ByVal board%, ByVal item%, ByVal value%) As Integer
        Ibconfig = Ibconfig32(BrdDev, board%, item%, value%)
        Call copy_globals()
    End Function

    Public Shared Function Ibcreate() As Integer
        Ibcreate = Ibcreate32()
        Call copy_globals()
    End Function

    Public Shared Function Ibload() As Integer
        Ibload = Ibload32()
        Call copy_globals()
    End Function

    Public Shared Function Ibsave() As Integer
        Ibsave = Ibsave32()
        Call copy_globals()
    End Function

    Public Shared Function Ibname(ByVal BrdDev%, ByVal board%, ByVal mode%, ByRef name$) As Integer
        Ibname = Ibname32(BrdDev%, board%, mode%, name$)
        Call copy_globals()
    End Function

    Public Shared Function IbEeprom(ByVal board%, ByVal mode%, ByRef eeprom$) As Integer
        IbEeprom = IbEeprom32(board%, mode%, eeprom$)
        Call copy_globals()
    End Function

    Public Shared Function IbSn(ByVal board%, ByVal mode%, ByRef sn$) As Integer
        IbSn = IbSn32(board%, mode%, sn$)
        Call copy_globals()
    End Function

    ' The following function is an ICS private function within the GPIB-32.DLL library.
    ' It has no arguments and no return value. Once it is called, the GPIB-32.DLL will
    ' force a reload of the GPIB.INI configuration. If it is called while anything is
    ' going on with the DLL, bad things may happen. Immediately after calling it, you
    ' should call one of the standard GPIB-32 functions. I recommend doing a SendIFC
    ' and you really should do it for all attached boards.
    Public Shared Sub LoadGPIBConfig()
        LoadGPIBConfig32()
    End Sub

    ' This checks to see if the GPIBDIR environment variable exists.
    ' Returns 0 if it does not exist.
    Public Shared Function TestGpibDirEnvVar() As Integer
        TestGpibDirEnvVar = TestGpibDirEnvVar32()
    End Function

    ' This checks to see if a GPIB.INI file exists in the GPIBDIR folder.
    ' Returns 0 if it does not exist.
    ' NOTE: This does not confirm validity of the file.
    Public Shared Function TestGpibIniFile() As Integer
        TestGpibIniFile = TestGpibIniFile32()
    End Function

    ' This checks to see if a GPIB-32.DLL exists and what type it is.
    ' Returns:
    '   0x0000 = none found
    '   0x00x1 = ICS, version 3 or better
    '   0x001x = found in current directory
    '   0x002x = found in Windows system directory
    '   0x004x = found in Windows directory
    '   0x008x = found in PATH directory
    Public Shared Function TestGpibLib() As Integer
        TestGpibLib = TestGpibLib32()
    End Function

End Class
