Public Class GPIB32Funcs

    '*********************************************************************
    '
    '  GPIB Library for Visual BASIC .NET
    '
    '  File: GPIB-32.VB                                        05-17-2006
    '
    ' This file contains the Visual Basic (32 bit) declarations for all
    ' GPIB library commands.   This file should be imported via Visual
    ' Basic's Load Text command (from the code menu).  It should be
    ' imported into either a form or the Public section of the program.
    '
    ' This file is ONLY for use with the Visual Basic 8 from Microsoft's 
    ' Visual Studio 2005.
    '
    '***********************************************************************


    'COMMON SHARED /GPIBVARS/ IBSTA%, IBERR%, IBCNT%, IBCNTL&
    Public Shared ibsta%, iberr%, ibcnt%, ibcntl&
    Public Shared GPIBInProgress%

    ' GPIB Commands
    Public Const GTL As Integer = &H1
    Public Const SDC As Integer = &H4
    Public Const PPC As Integer = &H5
    Public Const GETT As Integer = &H8
    Public Const TCT As Integer = &H9
    Public Const LLO As Integer = &H11
    Public Const DCL As Integer = &H14
    Public Const PPU As Integer = &H15
    Public Const SPE As Integer = &H18
    Public Const SPD As Integer = &H19
    Public Const UNL As Integer = &H3F
    Public Const UNT As Integer = &H5F
    Public Const PPE As Integer = &H60
    Public Const PPD As Integer = &H70


    ' Bit specifiers for ibsta status variable and wait mask
    Public Const EERR As Integer = &H8000  ' Error detected
    Public Const TIMO As Integer = &H4000  ' Timeout
    Public Const EEND As Integer = &H2000  ' EOI or EOS detected
    Public Const SRQI As Integer = &H1000  ' SRQ detected by CIC
    Public Const RQS As Integer = &H800    ' Device needs service
    Public Const SPOLL As Integer = &H400  ' Board has been serially polled
    Public Const eevent As Integer = &H200 ' An event has occured
    Public Const CMPL As Integer = &H100   ' I/O completed
    Public Const LOK As Integer = &H80     ' Local lockout state
    Public Const RREM As Integer = &H40    ' Remote state
    Public Const CIC As Integer = &H20     ' Controller-in-Charge
    Public Const AATN As Integer = &H10    ' Attention asserted
    Public Const TACS As Integer = &H8     ' Talker active
    Public Const LACS As Integer = &H4     ' Listener active
    Public Const DTAS As Integer = &H2     ' Device trigger state
    Public Const DCAS As Integer = &H1     ' Device clear state


    ' Error codes returned to IBERR variable
    Public Const EDVR As Integer = 0     ' DOS error
    Public Const ECIC As Integer = 1     ' Board must be CIC for this function
    Public Const ENOL As Integer = 2     ' No listeners detected
    Public Const EADR As Integer = 3     ' Board not addressed correctly
    Public Const EARG As Integer = 4     ' Invalid argument passed to function
    Public Const ESAC As Integer = 5     ' Function requires GPIB board to be SC
    Public Const EABO As Integer = 6     ' I/O operation aborted
    Public Const ENEB As Integer = 7     ' Invalid board specified
    Public Const EOIP As Integer = 10    ' I/O operation already running
    Public Const ECAP As Integer = 11    ' Board does not have requested capability
    Public Const EFSO As Integer = 12    ' Error returned from file system
    Public Const EBUS As Integer = 14    ' Command error on bus
    Public Const ESTB As Integer = 15    ' Serial poll response byte lost
    Public Const ESRQ As Integer = 16    ' SRQ is still asserted
    Public Const ETAB As Integer = 20    ' No device responding with ETAB

    Public Const EINT As Integer = 247   ' No interrupt configured on board.
    Public Const EWMD As Integer = 248   ' Windows is not in enhanced mode
    Public Const EVDD As Integer = 249   ' CBGPIB.386 is not installed
    Public Const EOVR As Integer = 250   ' Buffer overflow
    Public Const ESML As Integer = 251   ' Two library calls running simultaneously
    Public Const ECFG As Integer = 252   ' Board type doesn't match GPIB.CFG
    Public Const ETMR As Integer = 253   ' No Windows timers available
    Public Const ESLC As Integer = 254   ' No Windows selectors available
    Public Const EBRK As Integer = 255   ' Ctrl-Break pressed, exiting program


    ' EOS mode bits
    Public Const BIN As Integer = &H1000
    Public Const XEOS As Integer = &H800
    Public Const REOS As Integer = &H400


    ' Timeout code for ibtmo function
    Public Const TNONE As Integer = 0      ' No timeout
    Public Const T10us As Integer = 1      ' 10 usec
    Public Const T30us As Integer = 2      ' 30 usec
    Public Const T100us As Integer = 3     ' 100 usec
    Public Const T300us As Integer = 4     ' 300 usec
    Public Const T1ms As Integer = 5       ' 1 msec
    Public Const T3ms As Integer = 6       ' 3 msec
    Public Const T10ms As Integer = 7      ' 10 msec
    Public Const T30ms As Integer = 8      ' 30 msec
    Public Const T100ms As Integer = 9     ' 100 msec
    Public Const T300ms As Integer = 10    ' 300 msec
    Public Const T1s As Integer = 11       ' 1 sec
    Public Const T3s As Integer = 12       ' 3 sec
    Public Const T10s As Integer = 13      ' 10 sec
    Public Const T30s As Integer = 14      ' 30 sec
    Public Const T100s As Integer = 15     ' 100 sec
    Public Const T300s As Integer = 16     ' 300 sec
    Public Const T1000s As Integer = 17    ' 1000 sec

    '  IBLN Constants
    Public Const NO_SAD As Short = 0
    Public Const ALL_SAD As Short = -1


    ' Miscellaneous
    Public Const S As Integer = &H8             ' parallel poll bit
    Public Const LF As Integer = &HA            ' linefeed character
    ' Public Const NOADDR As Short = -1           'Terminator for address list
    Public Const NOADDR As Integer = -1           'Terminator for address list

    Public Const NULLend As Integer = 0         'Send() EOTMODE - Do nothing at the end of a transfer.
    Public Const NLend As Integer = 1           'Send() EOTMODE - Send NL with EOI after a transfer.
    Public Const DABend As Integer = 2          'Send() EOTMODE - Send EOI with the last DAB.
    Public Const STOPend As Integer = &H100     'Receive()( termination
    Public Const EventDTAS As Integer = 1       'used by IBEVENT()
    Public Const EventDCAS As Integer = 2       'used by IBEVENT()


    ' The following constants are used with ibconfig() to select a
    ' configurable option.
    Public Const IbcPAD As Integer = &H1             ' Primary address
    Public Const IbcSAD As Integer = &H2             ' Secondary address
    Public Const IbcTMO As Integer = &H3             ' Timeout
    Public Const IbcEOT As Integer = &H4             ' Send EOI with last byte
    Public Const IbcPPC As Integer = &H5             ' Parallel Poll Configure
    Public Const IbcREADDR As Integer = &H6          ' Repeat Addressing
    Public Const IbcAUTOPOLL As Integer = &H7        ' Disable automatic serial poll
    Public Const IbcCICPROT As Integer = &H8         ' Use CIC Protocol
    Public Const IbcIRQ As Integer = &H9             ' Interrupt level (or 0 for none)
    Public Const IbcSC As Integer = &HA              ' Board is system controller
    Public Const IbcSRE As Integer = &HB             ' Assert SRE for dev calls
    Public Const IbcEOSrd As Integer = &HC           ' Terminate read on EOS
    Public Const IbcEOSwrt As Integer = &HD          ' Send EOI with EOS
    Public Const IbcEOScmp As Integer = &HE          ' Use [7] or 8-bit compare with EOS
    Public Const IbcEOSchar As Integer = &HF         ' EOS character
    Public Const IbcPP2 As Integer = &H10            ' Use PP mode 2
    Public Const IbcTIMING As Integer = &H11         ' 1=2us, [2=500ns], 3=350ns
    Public Const IbcDMA As Integer = &H12            ' DMA channel (or 0 for none)
    Public Const IbcReadAdjust As Integer = &H13     ' Swap bytes on read
    Public Const IbcWriteAdjust As Integer = &H14    ' Swap bytes on write
    Public Const IbcEventQueue As Integer = &H15     ' Use event queue
    Public Const IbcSPollBit As Integer = &H16       ' Serial poll bit used
    Public Const IbcSendLLO As Integer = &H17        ' Automatically send LLO
    Public Const IbcSPollTime As Integer = &H18      ' Serial poll timeout
    Public Const IbcPPollTime As Integer = &H19      ' Parallel poll timeout
    Public Const IbcEndBitIsNormal As Integer = &H1A  ' Don't set end bit on EOS
    Public Const IbcBNA As Integer = &H200           ' A device's access board.
    Public Const IbcBaseAddr As Integer = &H201      ' A GPIB board's base I/O address.

    ' CBP config items - Don't exist in NI software
    Public Const IbcBoardType As Integer = &H300     ' Board type
    Public Const IbcChipType As Integer = &H301      ' ChipType
    Public Const IbcSlotNum As Integer = &H302       ' Slot Number
    Public Const IbcPCIIndex As Integer = &H303      ' PCI Index
    Public Const IbcBaseAdr2 As Integer = &H304      ' 2nd base address for PCI board
    Public Const IbcUseFIFOs As Integer = &H305      ' When to use FIFOs


    ' These are provided for compatability with NI's library
    Public Const IbaPAD As Integer = &H1
    Public Const IbaSAD As Integer = &H2
    Public Const IbaTMO As Integer = &H3
    Public Const IbaEOT As Integer = &H4
    Public Const IbaPPC As Integer = &H5
    Public Const IbaREADDR As Integer = &H6
    Public Const IbaAUTOPOLL As Integer = &H7
    Public Const IbaCICPROT As Integer = &H8
    Public Const IbaIRQ As Integer = &H9
    Public Const IbaSC As Integer = &HA
    Public Const IbaSRE As Integer = &HB
    Public Const IbaEOSrd As Integer = &HC
    Public Const IbaEOSwrt As Integer = &HD
    Public Const IbaEOScmp As Integer = &HE
    Public Const IbaEOSchar As Integer = &HF
    Public Const IbaPP2 As Integer = &H10
    Public Const IbaTIMING As Integer = &H11
    Public Const IbaDMA As Integer = &H12
    Public Const IbaReadAdjust As Integer = &H13
    Public Const IbaWriteAdjust As Integer = &H14
    Public Const IbaEventQueue As Integer = &H15
    Public Const IbaSPollBit As Integer = &H16
    Public Const IbaSendLLO As Integer = &H17
    Public Const IbaSPollTime As Integer = &H18
    Public Const IbaPPollTime As Integer = &H19
    Public Const IbaEndBitIsNormal As Integer = &H1A
    Public Const IbaBNA As Integer = &H200
    Public Const IbaBaseAddr As Integer = &H201

    ' CBI config items - Don't exist in NI software
    Public Const IbaBoardType As Integer = &H300
    Public Const IbaChipType As Integer = &H301
    Public Const IbaSlotNum As Integer = &H302
    Public Const IbaPCIIndex As Integer = &H303
    Public Const IbaBaseAdr2 As Integer = &H304
    Public Const IbaUseFIFOs As Integer = &H305


    ' These bits specify which lines can be monitored by IBLINES
    Public Const ValidDAV As Integer = &H1
    Public Const ValidNDAC As Integer = &H2
    Public Const ValidNRFD As Integer = &H4
    Public Const ValidIFC As Integer = &H8
    Public Const ValidREN As Integer = &H10
    Public Const ValidSRQ As Integer = &H20
    Public Const ValidATN As Integer = &H40
    Public Const ValidEOI As Integer = &H80

    ' These bits specify the current state of each GPIB line
    Public Const BusDAV As Integer = &H100
    Public Const BusNDAC As Integer = &H200
    Public Const BusNRFD As Integer = &H400
    Public Const BusIFC As Integer = &H800
    Public Const BusREN As Integer = &H1000
    Public Const BusSRQ As Integer = &H2000
    Public Const BusATN As Integer = &H4000
    Public Const BusEOI As Integer = &H8000

    ' GPIB-32.DLL routines
    Declare Function ibask32 Lib "gpib-32.dll" Alias "ibask" (ByVal board%, ByVal item%, ByRef value%) As Short
    Declare Function DLLibbna Lib "gpib-32.dll" Alias "ibbnaA" (ByVal Device%, ByVal boardname$) As Short
    Declare Function DLLibcac Lib "gpib-32.dll" Alias "ibcac" (ByVal Device%, ByVal sync%) As Short
    Declare Function DLLibclr Lib "gpib-32.dll" Alias "ibclr" (ByVal Device%) As Short
    Declare Function DLLibcmd Lib "gpib-32.dll" Alias "ibcmd" (ByVal board%, ByVal cmnd$, ByVal cmdcount%) As Short
    Declare Function DLLibcmda Lib "gpib-32.dll" Alias "ibcmda" (ByVal board%, ByVal cmnd$, ByVal cmdcount%) As Short
    Declare Function DLLibconfig Lib "gpib-32.dll" Alias "ibconfig" (ByVal board%, ByVal item%, ByVal value%) As Short
    Declare Function DLLibdev Lib "gpib-32.dll" Alias "ibdev" (ByVal boardindex%, ByVal pad%, ByVal sad%, ByVal timeout%, ByVal eot%, ByVal eos%) As Short
    Declare Function DLLibdma Lib "gpib-32.dll" Alias "ibdma" (ByVal board%, ByVal dma%) As Short
    Declare Function DLLibeos Lib "gpib-32.dll" Alias "ibeos" (ByVal boarddev%, ByVal eos%) As Short
    Declare Function DLLibeot Lib "gpib-32.dll" Alias "ibeot" (ByVal boarddev%, ByVal eot%) As Short
    Declare Function DLLibevent Lib "gpib-32.dll" Alias "ibevent" (ByVal board%, ByVal eevent%) As Short
    Declare Function DLLibfind Lib "gpib-32.dll" Alias "ibfindA" (ByVal devname$) As Short
    Declare Function DLLibgts Lib "gpib-32.dll" Alias "ibgts" (ByVal board%, ByVal handshake%) As Short
    Declare Function DLLibist Lib "gpib-32.dll" Alias "ibist" (ByVal board%, ByVal statusbit%) As Short
    Declare Function DLLibinit Lib "gpib-32.dll" Alias "ibinit" (ByVal board%) As Short
    Declare Function DLLiblines Lib "gpib-32.dll" Alias "iblines" (ByVal board%, ByRef clines As Short) As Short
    Declare Function DLLibln Lib "gpib-32.dll" Alias "ibln" (ByVal board%, ByVal pad%, ByVal sad%, ByRef listenflag As Short) As Short
    Declare Function DLLibloc Lib "gpib-32.dll" Alias "ibloc" (ByVal boarddev%) As Short
    Declare Function DLLibonl Lib "gpib-32.dll" Alias "ibonl" (ByVal boarddev%, ByVal online%) As Short
    Declare Function DLLibpad Lib "gpib-32.dll" Alias "ibpad" (ByVal boarddev%, ByVal address%) As Short
    Declare Function DLLibpct Lib "gpib-32.dll" Alias "ibpct" (ByVal Device%) As Short
    Declare Function DLLibppc Lib "gpib-32.dll" Alias "ibppc" (ByVal boarddev%, ByVal Cmd%) As Short
    Declare Function ibptrs Lib "gpib-32.dll" Alias "ibptrs" (ByVal ibstaPtr%, ByVal iberrPtr%, ByVal ibcntPtr%, ByVal ibcntlPtr&) As Short
    Declare Function DLLibrd Lib "gpib-32.dll" Alias "ibrd" (ByVal boardev%, ByVal readstr$, ByVal rdcount As Integer) As Integer
    Declare Function DLLibrda Lib "gpib-32.dll" Alias "ibrda" (ByVal boardev%, ByVal readstr$, ByVal rdcount%) As Integer
    Declare Function DLLibrdf Lib "gpib-32.dll" Alias "ibrdfA" (ByVal boardev%, ByVal filename$) As Integer
    Declare Function DLLibrdi Lib "gpib-32.dll" Alias "ibrdi" (ByVal boarddev%, ByVal intarray%, ByVal count As Integer) As Integer
    Declare Function DLLibrdia Lib "gpib-32.dll" Alias "ibrdia" (ByVal boarddev%, ByVal intarray%, ByVal count As Integer) As Integer
    Declare Function DLLibrpp Lib "gpib-32.dll" Alias "ibrpp" (ByVal boarddev%, ByRef response As Char) As Short
    Declare Function DLLibrsc Lib "gpib-32.dll" Alias "ibrsc" (ByVal board%, ByVal ctrl%) As Short
    Declare Function DLLibrsp Lib "gpib-32.dll" Alias "ibrsp" (ByVal Device%, ByRef serialpollbyte As Char) As Short
    Declare Function DLLibrsv Lib "gpib-32.dll" Alias "ibrsv" (ByVal boarddev%, ByVal statusbyte%) As Short
    Declare Function DLLibsad Lib "gpib-32.dll" Alias "ibsad" (ByVal boarddev%, ByVal address%) As Short
    Declare Function DLLibsic Lib "gpib-32.dll" Alias "ibsic" (ByVal board%) As Short
    Declare Function DLLibsre Lib "gpib-32.dll" Alias "ibsre" (ByVal board%, ByVal ren%) As Short
    Declare Function DLLibstop Lib "gpib-32.dll" Alias "ibstop" (ByVal boarddev%) As Short
    Declare Function DLLibtmo Lib "gpib-32.dll" Alias "ibtmo" (ByVal boarddev%, ByVal timeout%) As Short
    Declare Function DLLibtrg Lib "gpib-32.dll" Alias "ibtrg" (ByVal Device%) As Short
    Declare Function DLLibwait Lib "gpib-32.dll" Alias "ibwait" (ByVal boarddev%, ByVal mask%) As Short
    Declare Function DLLibwrt Lib "gpib-32.dll" Alias "ibwrt" (ByVal boarddev%, ByVal writestr$, ByVal strcount As Integer) As Integer
    Declare Function DLLibwrta Lib "gpib-32.dll" Alias "ibwrta" (ByVal boarddev%, ByVal writestr$, ByVal strcount As Integer) As Integer
    Declare Function DLLibwrtf Lib "gpib-32.dll" Alias "ibwrtfA" (ByVal boarddev%, ByVal filename$) As Integer
    Declare Function DLLibwrti Lib "gpib-32.dll" Alias "ibwrti" (ByVal boarddev%, ByVal intarray%, ByVal count As Integer) As Integer
    Declare Function DLLibwrtia Lib "gpib-32.dll" Alias "ibwrtia" (ByVal boarddev%, ByVal intarray%, ByVal count As Integer) As Integer

    Declare Sub DLLAllSpoll Lib "gpib-32.dll" Alias "AllSpoll" (ByVal dev%, ByRef adrlist As Short, ByRef results As Short)
    Declare Sub DLLDevClear Lib "gpib-32.dll" Alias "DevClear" (ByVal dev%, ByVal adr As Short)
    Declare Sub DLLDevClearList Lib "gpib-32.dll" Alias "DevClearList" (ByVal ByValdev%, ByRef adrlist As Short)
    Declare Sub DLLEnableLocal Lib "gpib-32.dll" Alias "EnableLocal" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLEnableRemote Lib "gpib-32.dll" Alias "EnableRemote" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLFindLstn Lib "gpib-32.dll" Alias "FindLstn" (ByVal dev%, ByRef adrlist As Short, ByRef results As Short, ByVal limit%)
    Declare Sub DLLFindRQS Lib "gpib-32.dll" Alias "FindRQS" (ByVal dev%, ByRef adrlist As Short, ByRef result%)
    Declare Sub DLLPassControl Lib "gpib-32.dll" Alias "PassControl" (ByVal dev%, ByVal adr%)
    Declare Sub DLLPPoll Lib "gpib-32.dll" Alias "PPoll" (ByVal dev%, ByRef result As Short)
    Declare Sub DLLPPollConfig Lib "gpib-32.dll" Alias "PollConfig" (ByVal dev%, ByVal adr%, ByVal lline%, ByVal sense%)
    Declare Sub DLLPPollUnconfig Lib "gpib-32.dll" Alias "PollUnconfig" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLReadStatusByte Lib "gpib-32.dll" Alias "ReadStatusByte" (ByVal dev%, ByVal adr%, ByRef result As Short)
    Declare Sub DLLRcvRespMsg Lib "gpib-32.dll" Alias "RcvRespMsg" (ByVal adr%, ByVal buf$, ByVal length%, ByVal term%)
    Declare Sub DLLReceive Lib "gpib-32.dll" Alias "Receive" (ByVal dev%, ByVal adr%, ByVal buf$, ByVal count%, ByVal term%)
    Declare Sub DLLReceiveSetup Lib "gpib-32.dll" Alias "ReceiveSetup" (ByVal dev%, ByVal adr%)
    Declare Sub DLLResetSys Lib "gpib-32.dll" Alias "ResetSys" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLSend Lib "gpib-32.dll" Alias "Send" (ByVal dev%, ByVal adr%, ByVal buf$, ByVal length%, ByVal term%)
    Declare Sub DLLSendDataBytes Lib "gpib-32.dll" Alias "SendDataBytes" (ByVal board As Integer, ByVal buf$, ByVal length As Integer, ByVal term As Integer)
    Declare Sub DLLSendCmds Lib "gpib-32.dll" Alias "SendCmds" (ByVal dev%, ByVal cmdbuf$, ByVal length&)
    Declare Sub DLLSendIFC Lib "gpib-32.dll" Alias "SendIFC" (ByVal dev%)
    Declare Sub DLLSendList Lib "gpib-32.dll" Alias "SendList" (ByVal dev%, ByRef adr As Short, ByVal buf$, ByVal length%, ByVal term%)
    Declare Sub DLLSendLLO Lib "gpib-32.dll" Alias "SendLLO" (ByVal dev%)
    Declare Sub DLLSendSetup Lib "gpib-32.dll" Alias "SendSetup" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLSetRWLS Lib "gpib-32.dll" Alias "SetRWLS" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLTestSRQ Lib "gpib-32.dll" Alias "TestSRQ" (ByVal dev%, ByRef result As Short)
    Declare Sub DLLTestSys Lib "gpib-32.dll" Alias "TestSys" (ByVal dev%, ByRef adrlist As Short, ByRef results As Short)
    Declare Sub DLLTrigger Lib "gpib-32.dll" Alias "Trigger" (ByVal dev%, ByVal adr%)
    Declare Sub DLLTriggerList Lib "gpib-32.dll" Alias "TriggerList" (ByVal dev%, ByRef adrlist As Short)
    Declare Sub DLLWaitSRQ Lib "gpib-32.dll" Alias "WaitSRQ" (ByVal dev%, ByRef result As Short)
    Declare Function DLLGetIBVars Lib "gpib-32.dll" (ByRef ibsta%, ByRef iberr%, ByRef ibcnt%, ByRef ibcntl&) As Integer
    Declare Function ThreadIbsta Lib "gpib-32.dll" Alias "ThreadIbsta" () As Integer
    Declare Function ThreadIberr Lib "gpib-32.dll" Alias "ThreadIberr" () As Integer
    Declare Function ThreadIbcnt Lib "gpib-32.dll" Alias "ThreadIbcnt" () As Integer
    Declare Function ThreadIbcntl Lib "gpib-32.dll" Alias "ThreadIbcntl" () As Long

    '************************
    ' 488.1 Functions
    '************************

    Public Shared Function ilask(ByVal dev%, ByVal opt%, ByVal rval%) As Integer
        ilask% = ibask32(dev%, opt%, rval%)
        Call copy_globals()
    End Function

    Public Shared Function ilbna(ByVal dev%, ByVal devname$) As Integer
        ilbna% = DLLibbna(dev%, devname$)
        Call copy_globals()
    End Function

    Public Shared Function ilcac(ByVal dev%, ByVal v%) As Integer
        ilcac% = DLLibcac(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilclr(ByVal dev%) As Integer
        ilclr% = DLLibclr(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilcmd(ByVal dev%, ByVal buf$, ByVal cnt&) As Integer
        ilcmd% = DLLibcmd(dev%, buf$, cnt&)
        Call copy_globals()
    End Function

    Public Shared Function ilcmda(ByVal dev%, ByVal buf$, ByVal cnt&) As Integer
        ilcmda% = DLLibcmd(dev%, buf$, cnt&)
        Call copy_globals()
    End Function

    Public Shared Function ilconfig(ByVal bdid%, ByVal opt%, ByVal v%) As Integer
        ilconfig% = DLLibconfig(bdid%, opt%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ildev(ByVal bdid%, ByVal pad%, ByVal sad%, ByVal tmo%, ByVal eot%, ByVal eos%) As Integer
        ildev% = DLLibdev(bdid%, pad%, sad%, tmo%, eot%, eos%)
        Call copy_globals()
    End Function

    Public Shared Function ildma(ByVal dev%, ByVal v%) As Integer
        ildma% = DLLibdma(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ileos(ByVal dev%, ByVal v%) As Integer
        ileos% = DLLibeos(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ileot(ByVal dev%, ByVal v%) As Integer
        ileot% = DLLibeot(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilevent(ByVal dev%, ByVal eevent%) As Integer
        ilevent% = DLLibevent(dev%, eevent%)
        Call copy_globals()
    End Function

    Public Shared Function ilfind(ByVal devname$) As Integer
        ilfind% = DLLibfind(devname$)
        Call copy_globals()
    End Function

    Public Shared Function ilgts(ByVal dev%, ByVal v%) As Integer
        ilgts% = DLLibgts(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilinit(ByVal dev%) As Integer
        ilinit% = DLLibinit(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilist(ByVal dev%, ByVal v%) As Integer
        ilist% = DLLibist(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function illines(ByVal dev%, ByRef lines As Short) As Short
        illines = DLLiblines(dev%, lines)
        Call copy_globals()
    End Function

    Public Shared Function illn(ByVal dev%, ByVal pad%, ByVal sad%, ByVal listener As Short) As Integer
        illn% = DLLibln(dev%, pad%, sad%, listener)
        Call copy_globals()
    End Function

    Public Shared Function illoc(ByVal dev%) As Integer
        illoc% = DLLibloc(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilonl(ByVal dev%, ByVal v%) As Integer
        ilonl% = DLLibonl(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilpad(ByVal dev%, ByVal v%) As Integer
        ilpad% = DLLibpad(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilpct(ByVal dev%) As Integer
        ilpct% = DLLibpct(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilppc(ByVal dev%, ByVal v%) As Integer
        ilppc% = DLLibppc(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilrd(ByVal dev%, ByVal buf$, ByVal cnt%) As Integer
        ilrd% = DLLibrd(dev%, buf$, cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilrda(ByVal dev%, ByVal buf$, ByVal cnt%) As Integer
        ilrda% = DLLibrd(dev%, buf$, cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilrdf(ByVal dev%, ByVal filename$) As Integer
        ilrdf% = DLLibrdf(dev%, filename$)
        Call copy_globals()
    End Function

    Public Shared Function ilrdi(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        ilrdi% = DLLibrdi(dev%, ibuf%(0), cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilrdia(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        ilrdia% = DLLibrdi(dev%, ibuf%(0), cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilrpp(ByVal dev%, ByRef ppr%) As Integer
        Dim c As Char
        ilrpp% = DLLibrpp(dev%, c)
        ppr = Microsoft.VisualBasic.Val(c)
        Call copy_globals()
    End Function

    Public Shared Function ilrsc(ByVal dev%, ByVal v%) As Integer
        ilrsc% = DLLibrsc(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilrsp(ByVal dev%, ByVal rsp%) As Integer
        Dim c As Char
        ilrsp% = DLLibrsp(dev%, c)
        rsp = Microsoft.VisualBasic.Val(c)
        Call copy_globals()
    End Function

    Public Shared Function ilrsv(ByVal dev%, ByVal v%) As Integer
        ilrsv% = DLLibrsv(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilsad(ByVal dev%, ByVal v%) As Integer
        ilsad% = DLLibsad(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilsic(ByVal dev%) As Integer
        ilsic% = DLLibsic(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilsre(ByVal dev%, ByVal v%) As Integer
        ilsre% = DLLibsre(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function ilstop(ByVal dev%) As Integer
        ilstop% = DLLibstop(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ILTMO(ByVal dev%, ByVal v%) As Integer
        ILTMO% = DLLibtmo(dev%, v%)
        Call copy_globals()
    End Function

    Public Shared Function iltrg(ByVal dev%) As Integer
        iltrg% = DLLibtrg(dev%)
        Call copy_globals()
    End Function

    Public Shared Function ilwait(ByVal dev%, ByVal mask%) As Integer
        ilwait% = DLLibwait(dev%, mask)
        Call copy_globals()
    End Function

    Public Shared Function ilwrt(ByVal dev%, ByVal buf$, ByVal cnt%) As Integer
        ilwrt% = DLLibwrt(dev%, buf$, cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilwrta(ByVal dev%, ByVal buf$, ByVal cnt%) As Integer
        ilwrta% = DLLibwrt(dev%, buf$, cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilwrtf(ByVal dev%, ByVal filename$) As Integer
        ilwrtf% = DLLibwrtf(dev%, filename$)
        Call copy_globals()
    End Function

    Public Shared Function ilwrti(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        ilwrti% = DLLibwrti(dev%, ibuf%(0), cnt%)
        Call copy_globals()
    End Function

    Public Shared Function ilwrtia(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        ilwrtia% = DLLibwrti(dev%, ibuf%(0), cnt%)
        Call copy_globals()
    End Function


    '************************
    ' 488.1 Subroutines
    '************************

    Public Shared Function ibask(ByVal dev%, ByVal opt%, ByRef rval%) As Short
        Dim a As Short
        a = ibask32(dev%, opt%, rval%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibbna(ByVal dev%, ByVal devname$) As Short
        Dim a As Short
        a = DLLibbna(dev%, devname$)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibcac(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibcac(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibclr(ByVal dev%) As Short
        Dim a As Short
        a = DLLibclr(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibcmd(ByVal dev%, ByVal buf$) As Short
        Dim a As Short
        a = DLLibcmd(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibcmda(ByVal dev%, ByVal buf$) As Short
        Dim a As Short
        a = DLLibcmd(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibconfig(ByVal bdid%, ByVal opt%, ByVal v%) As Short
        Dim a As Short
        a = DLLibconfig(bdid%, opt%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibdev(ByVal bdid%, ByVal pad%, ByVal sad%, ByVal tmo%, ByVal eot%, ByVal eos%, ByRef dev As Short) As Short
        dev = DLLibdev(bdid%, pad%, sad%, tmo%, eot%, eos%)
        Call copy_globals()
        Return dev
    End Function

    Public Shared Function ibdma(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibdma(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibeos(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibeos(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibeot(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibeot(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibevent(ByVal dev%, ByVal eevent%) As Short
        Dim a As Short
        a = DLLibevent(dev%, eevent%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibfind(ByVal devname$, ByRef dev%) As Short
        dev = DLLibfind(devname$)
        ibfind = dev
        Call copy_globals()
        Return dev
    End Function

    Public Shared Function ibgts(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibgts(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibinit(ByVal dev%) As Short
        Dim a As Short
        a = DLLibinit(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibist(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibist(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function iblines(ByVal dev%, ByRef lines As Short) As Short
        Dim a As Short
        a = DLLiblines(dev%, lines)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibln(ByVal dev%, ByVal pad%, ByVal sad%, ByRef listener As Short) As Short
        Dim a As Short
        a = DLLibln(dev%, pad%, sad%, listener)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibloc(ByVal dev%) As Short
        Dim a As Short
        a = DLLibloc(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibonl(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibonl(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibpad(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibpad(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibpct(ByVal dev%) As Short
        Dim a As Short
        a = DLLibpct(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibppc(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibppc(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibrd(ByVal dev%, ByRef buf$) As Integer
        Dim a%
        a% = DLLibrd(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibrda(ByVal dev%, ByRef buf$) As Integer
        Dim a%
        a% = DLLibrd(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibrdf(ByVal dev%, ByVal filename$) As Integer
        Dim a%
        a% = DLLibrdf(dev%, filename$)
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibrdi(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        Dim a%
        a% = DLLibrdi(dev%, ibuf%(0), cnt%)
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibrdia(ByVal dev%, ByVal ibuf%(), ByVal cnt%) As Integer
        Dim a%
        a% = DLLibrdi(dev%, ibuf%(0), cnt%)
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibrpp(ByVal dev%, ByRef ppr%) As Short
        Dim a As Short
        Dim c As Char
        a = DLLibrpp(dev%, c)
        ppr% = Microsoft.VisualBasic.Val(c)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibrsc(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibrsc(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibrsp(ByVal dev%, ByVal rsp%) As Short
        Dim a As Short
        Dim c As Char
        a = DLLibrsp(dev%, c)
        rsp = Microsoft.VisualBasic.Val(c)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibrsv(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibrsv(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibsad(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibsad(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibsic(ByVal dev%) As Short
        Dim a As Short
        a = DLLibsic(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibsre(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibsre(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibstop(ByVal dev%) As Short
        Dim a As Short
        a = DLLibstop(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibtmo(ByVal dev%, ByVal v%) As Short
        Dim a As Short
        a = DLLibtmo(dev%, v%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibtrg(ByVal dev%) As Short
        Dim a As Short
        a = DLLibtrg(dev%)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibwait(ByVal dev%, ByVal mask%) As Short
        Dim a As Short
        a = DLLibwait(dev%, mask)
        Call copy_globals()
        Return a
    End Function

    Public Shared Function ibwrt(ByVal dev%, ByVal buf$) As Integer
        Dim a%
        a% = DLLibwrt(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibwrta(ByVal dev%, ByVal buf$) As Integer
        Dim a%
        a% = DLLibwrt(dev%, buf$, Len(buf$))
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibwrtf(ByVal dev%, ByVal filename$) As Integer
        Dim a%
        a% = DLLibwrtf(dev%, filename$)
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibwrti(ByVal dev%, ByVal ibuf%(), ByVal cnt&) As Integer
        Dim a%
        a% = DLLibwrti(dev%, ibuf%(0), cnt&)
        Call copy_globals()
        Return a%
    End Function

    Public Shared Function ibwrtia(ByVal dev%, ByVal ibuf%(), ByVal cnt&) As Integer
        Dim a%
        a% = DLLibwrti(dev%, ibuf%(0), cnt&)
        Call copy_globals()
        Return a%
    End Function



    '************************
    ' 488.2 Subroutines
    '************************
    Public Shared Sub copy_globals()
        'Call DLLGetIBVars(ibsta%, iberr%, ibcnt%, ibcntl&)
        ibsta% = ThreadIbsta()
        iberr% = ThreadIberr()
        ibcnt% = ThreadIbcnt()
        ibcntl& = 0
        ibcntl& = ThreadIbcntl() And &HFFFFFFFF&
    End Sub

    Public Shared Sub AllSpoll(ByVal dev%, ByRef adrlist() As Short, ByRef results() As Short)
        Call DLLAllSpoll(dev%, adrlist(0), results(0))
        Call copy_globals()
    End Sub

    Public Shared Sub DevClear(ByVal dev%, ByVal adr As Short)
        Call DLLDevClear(dev%, adr)
        Call copy_globals()
    End Sub

    Public Shared Sub DevClearList(ByVal ByValdev%, ByRef adrlist() As Short)
        Call DLLDevClearList(ByValdev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub EnableLocal(ByVal dev%, ByRef adrlist() As Short)
        Call DLLEnableLocal(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub EnableRemote(ByVal dev%, ByVal adrlist() As Short)
        Call DLLEnableRemote(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub FindLstn(ByVal dev%, ByRef adrlist() As Short, ByRef results() As Short, ByVal limit%)
        Call DLLFindLstn(dev%, adrlist(0), results(0), limit%)
        Call copy_globals()
    End Sub

    Public Shared Sub FindRQS(ByVal dev%, ByVal adrlist() As Short, ByRef result%)
        Call DLLFindRQS(dev%, adrlist(0), result%)
        Call copy_globals()
    End Sub

    Public Shared Sub PassControl(ByVal dev%, ByVal adr%)
        Call DLLPassControl(dev%, adr%)
        Call copy_globals()
    End Sub

    Public Shared Sub PPoll(ByVal dev%, ByVal result As Short)
        Call DLLPPoll(dev%, result)
        Call copy_globals()
    End Sub

    Public Shared Sub PPollConfig(ByVal dev%, ByVal adr%, ByVal lline%, ByVal sense%)
        Call DLLPPollConfig(dev%, adr%, lline%, sense%)
        Call copy_globals()
    End Sub

    Public Shared Sub PPollUnconfig(ByVal dev%, ByVal adrlist() As Short)
        Call DLLPPollUnconfig(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub RcvRespMsg(ByVal board%, ByRef buf$, ByVal term%)
        Dim length%
        length% = Len(buf$)
        Call DLLRcvRespMsg(board%, buf$, length%, term%)
        Call copy_globals()
    End Sub

    Public Shared Sub ReadStatusByte(ByVal dev%, ByVal adr%, ByRef result As Short)
        Call DLLReadStatusByte(dev%, adr%, result)
        Call copy_globals()
    End Sub

    Public Shared Sub Receive(ByVal dev%, ByVal adr%, ByRef buf$, ByVal term%)
        Dim length%
        length% = Len(buf$)
        Call DLLReceive(dev%, adr%, buf$, length%, term%)
        Call copy_globals()
    End Sub

    Public Shared Sub ReceiveSetup(ByVal dev%, ByVal adr%)
        Call DLLReceiveSetup(dev%, adr%)
        Call copy_globals()
    End Sub

    Public Shared Sub ResetSys(ByVal dev%, ByRef adrlist() As Short)
        Call DLLResetSys(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub Send(ByVal dev%, ByVal adr%, ByVal buf$, ByVal term%)
        Dim length%
        length% = Len(buf$)
        Call DLLSend(dev%, adr%, buf$, length%, term%)
        Call copy_globals()
    End Sub

    Public Shared Sub SendCmds(ByVal dev%, ByVal cmdbuf$)
        Dim length&
        length& = Len(cmdbuf$)
        Call DLLSendCmds(dev%, cmdbuf$, length&)
        Call copy_globals()
    End Sub

    Public Shared Sub SendDataBytes(ByVal board%, ByVal buf$, ByVal term%)
        Dim length As Integer
        length = Len(buf$)
        Call DLLSendDataBytes(board%, buf$, length, term%)
        Call copy_globals()
    End Sub

    Public Shared Sub SendIFC(ByVal dev%)
        Call DLLSendIFC(dev%)
        Call copy_globals()
    End Sub

    Public Shared Sub SendList(ByVal dev%, ByRef adr() As Short, ByVal buf$, ByVal term%)
        Dim length%
        length% = Len(buf$)
        Call DLLSendList(dev%, adr(0), buf$, length%, term%)
        Call copy_globals()
    End Sub

    Public Shared Sub SendLLO(ByVal dev%)
        Call DLLSendLLO(dev%)
        Call copy_globals()
    End Sub

    Public Shared Sub SendSetup(ByVal dev%, ByVal adrlist() As Short)
        Call DLLSendSetup(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub SetRWLS(ByVal dev%, ByVal adrlist() As Short)
        Call DLLSetRWLS(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub TestSRQ(ByVal dev%, ByRef result As Short)
        Call DLLTestSRQ(dev%, result)
        Call copy_globals()
    End Sub

    Public Shared Sub TestSys(ByVal dev%, ByRef adrlist() As Short, ByRef results() As Short)
        Call DLLTestSys(dev%, adrlist(0), results(0))
        Call copy_globals()
    End Sub

    Public Shared Sub Trigger(ByVal dev%, ByVal adr%)
        Call DLLTrigger(dev%, adr%)
        Call copy_globals()
    End Sub

    Public Shared Sub TriggerList(ByVal dev%, ByRef adrlist() As Short)
        Call DLLTriggerList(dev%, adrlist(0))
        Call copy_globals()
    End Sub

    Public Shared Sub WaitSRQ(ByVal dev%, ByRef result As Short)
        Call DLLWaitSRQ(dev%, result)
        Call copy_globals()
    End Sub

End Class
